/*  
 * Copyright (c) 2011 Ant Kutschera
 * 
 * This file is part of Ant Kutschera's blog, 
 * http://blog.maxant.co.uk
 * 
 * This is free software: you can redistribute
 * it and/or modify it under the terms of the
 * Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 * 
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the Lesser GNU General Public License for
 * more details. 
 * 
 * You should have received a copy of the
 * Lesser GNU General Public License along with this software.
 * If not, see http://www.gnu.org/licenses/.
 */
package ch.maxant.blog.nio.websockets;

import java.io.IOException;
import java.math.BigDecimal;

import javax.ejb.EJB;
import javax.servlet.annotation.HttpConstraint;
import javax.servlet.annotation.ServletSecurity;
import javax.servlet.annotation.WebServlet;
import javax.servlet.websockets.EventType.EventSubType;
import javax.servlet.websockets.WebSocketsServlet;
import javax.servlet.websockets.WebSocketsServletRequest;
import javax.servlet.websockets.WebSocketsServletResponse;
import javax.servlet.websockets.WebSocketsSession;
import javax.xml.ws.WebServiceRef;

import ch.maxant.blog.nio.data.CommandContainer;
import ch.maxant.blog.nio.ejbs.SalesLog;
import ch.maxant.blog.nio.webservices.PricingSystem;

/**
 * a demo of what a web socket servlet could look like.
 * it uses resources, security and JSON binding!
 */
@WebServlet(urlPatterns={"/TestServlet"})
@ServletSecurity(@HttpConstraint(rolesAllowed = "registered"))
public class TestServlet extends WebSocketsServlet<CommandContainer> {
	private static final long serialVersionUID = 1L;

	@WebServiceRef
	private PricingSystem pricingSystem;
	
	@EJB
	private SalesLog salesLog;
	
	@Override
	protected void doOpen(WebSocketsServletRequest<CommandContainer> request) {
		System.out.println("connected to session " + request.getSession().getId());
	}

	@Override
	protected void doJSONMessage(WebSocketsServletRequest<CommandContainer> request,
			WebSocketsServletResponse response) throws IOException {

		CommandContainer c = request.getDataObject();
		
		if("CLOSE".equals(c.getCommand())){
		
			System.out.println("closing connection to client");
			response.close();
			
			actualClose(EventSubType.SESSION_END, request.getSession());
			
			return;

		}else if("PRICE".equals(c.getCommand())){
		
			c.setResult(pricingSystem.getPrice(c.getData()).toString());

		}else if("BUY".equals(c.getCommand())){
			
			//e.g. GOOG,100@200.25
			int idx = c.getData().indexOf(',');
			String shareCode = c.getData().substring(0, idx);
			idx = c.getData().indexOf('@');
			String numShares = c.getData().substring(shareCode.length()+1, idx);
			String price = c.getData().substring(idx+1, c.getData().length()-1);

			salesLog.logPurchase(shareCode, new BigDecimal(price), Integer.parseInt(numShares));
			
			c.setResult("sold " + numShares + " of " + shareCode + " for " + request.getUserPrincipal().getName());

		}else{
			
			c.setResult("unknown command " + c.getCommand());
			
		}

		System.out.println("handled command " + c.getCommand() + " on session " + request.getSession().getId() + ": " + c.getResult());

		response.sendJSONResponse(c);
	}

	@Override
	protected void doError(EventSubType eventSubType, WebSocketsSession session, Exception e) {

		System.out.println("error: " + eventSubType + " on session " + session.getId() + ".  ex was: " + e);
	}

	@Override
	public void doClose(EventSubType eventSubType, WebSocketsSession session) {
		actualClose(eventSubType, session);
	}
	
	private void actualClose(EventSubType eventSubType, WebSocketsSession session) {
		System.out.println("closed session " + session.getId() + ": " + eventSubType);
	}

}
